"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_fetch_1 = require("node-fetch");
const querystring_1 = require("querystring");
const provider_1 = require("../provider");
const info_1 = require("../info");
class DiscordProfile {
    constructor(profile) {
        Object.assign(this, profile);
    }
    getUniqueId() {
        return this.id;
    }
}
exports.DiscordProfile = DiscordProfile;
class DiscordProvider extends provider_1.Provider {
    constructor(clientId, clientSecret, scopes = []) {
        super();
        this.clientId = clientId;
        this.clientSecret = clientSecret;
        this.scopes = scopes;
        this.name = 'discord';
        this.tokenUrl = 'https://discordapp.com/api/oauth2/token';
        this.authUrl = 'https://discordapp.com/api/oauth2/authorize';
        this.profileUrl = 'https://discordapp.com/api/users/@me';
    }
    requestToken(code, redirect_uri) {
        return node_fetch_1.default(this.tokenUrl, {
            method: 'POST',
            headers: {
                'Accept-Encoding': 'gzip,deflate',
                'Content-Type': 'application/x-www-form-urlencoded',
            },
            body: querystring_1.stringify({
                code,
                redirect_uri,
                client_id: this.clientId,
                client_secret: this.clientSecret,
                grant_type: 'authorization_code',
            }),
            compress: true,
        })
            .then(res => res.json())
            .then(token => token);
    }
    getProfile(tokens) {
        return node_fetch_1.default(this.profileUrl, {
            headers: {
                Authorization: `Bearer ${tokens.access_token}`,
                'Accept-Encoding': 'gzip,deflate',
                'User-Agent': `${info_1.name} (https://github.com/UnwrittenFun/hapi-oauth, ${info_1.version})`,
            },
            compress: true,
        })
            .then(res => res.json())
            .then(profile => new DiscordProfile(profile));
    }
}
exports.DiscordProvider = DiscordProvider;
