"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_fetch_1 = require("node-fetch");
const querystring_1 = require("querystring");
const provider_1 = require("../provider");
const info_1 = require("../info");
class MicrosoftProfile {
    constructor(profile) {
        Object.assign(this, profile);
    }
    getUniqueId() {
        return this.id;
    }
}
exports.MicrosoftProfile = MicrosoftProfile;
class MicrosoftProvider extends provider_1.Provider {
    constructor(clientId, clientSecret, scopes = []) {
        super();
        this.clientId = clientId;
        this.clientSecret = clientSecret;
        this.scopes = scopes;
        this.name = "microsoft";
        this.tenant = "common";
        this.tokenUrl = `https://login.microsoftonline.com/${this.tenant}/oauth2/v2.0/token`;
        this.authUrl = `https://login.microsoftonline.com/${this.tenant}/oauth2/v2.0/authorize`;
        this.profileUrl = "https://graph.microsoft.com/v1.0/me";
    }
    requestToken(code, redirect_uri) {
        return node_fetch_1.default(this.tokenUrl, {
            method: "POST",
            headers: {
                "Accept-Encoding": "gzip,deflate",
                "Content-Type": "application/x-www-form-urlencoded"
            },
            body: querystring_1.stringify({
                code,
                redirect_uri,
                client_id: this.clientId,
                client_secret: this.clientSecret,
                grant_type: "authorization_code"
            }),
            compress: true
        })
            .then(res => res.json())
            .then(token => token);
    }
    getProfile(tokens) {
        return node_fetch_1.default(this.profileUrl, {
            headers: {
                Authorization: `Bearer ${tokens.access_token}`,
                "Accept-Encoding": "gzip,deflate",
                "User-Agent": `${info_1.name} (https://github.com/UnwrittenFun/hapi-oauth, ${info_1.version})`
            },
            compress: true
        })
            .then(res => res.json())
            .then(profile => {
            return new MicrosoftProfile(profile);
        });
    }
}
exports.MicrosoftProvider = MicrosoftProvider;
