"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Wreck = require("wreck");
const qs = require("querystring");
const node_fetch_1 = require("node-fetch");
const provider_1 = require("../provider");
class PicartoProfile {
    constructor(profile) {
        Object.assign(this, profile);
    }
    getUniqueId() {
        return this.channel_details.user_id;
    }
}
exports.PicartoProfile = PicartoProfile;
class PicartoProvider extends provider_1.Provider {
    constructor(clientId, clientSecret, scopes) {
        super();
        this.clientId = clientId;
        this.clientSecret = clientSecret;
        this.scopes = scopes;
        this.name = 'picarto';
        this.tokenUrl = 'https://oauth.picarto.tv/token';
        this.authUrl = 'https://oauth.picarto.tv/authorize';
        this.profileUrl = 'https://api.picarto.tv/v1/user';
    }
    requestToken(code, redirect_uri) {
        return new Promise((resolve, reject) => {
            const query = {
                code,
                redirect_uri,
                client_id: this.clientId,
                client_secret: this.clientSecret,
                grant_type: 'authorization_code',
            };
            Wreck.post(`${this.tokenUrl}?${qs.stringify(query)}`, {
                json: true,
            }, (err, message, res) => {
                if (err) {
                    reject(err);
                    return;
                }
                if (res.error) {
                    const error = new Error(res.error_description);
                    error.name = err.error;
                    reject(error);
                    return;
                }
                resolve(res);
            });
        });
    }
    getProfile(tokens) {
        return node_fetch_1.default(this.profileUrl, {
            headers: {
                Authorization: `Bearer ${tokens.access_token}`,
            },
        })
            .then(res => res.json())
            .then(profile => new PicartoProfile(profile));
    }
}
exports.PicartoProvider = PicartoProvider;
