"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Wreck = require("wreck");
const qs = require("querystring");
const provider_1 = require("../provider");
class SmashcastProvider extends provider_1.Provider {
    constructor(clientId, clientSecret) {
        super();
        this.clientId = clientId;
        this.clientSecret = clientSecret;
        this.name = 'smashcast';
        this.tokenUrl = 'https://api.smashcast.tv/oauth/exchange';
        this.authUrl = 'https://api.smashcast.tv/oauth/login';
        this.hash = new Buffer(`${clientId} ${clientSecret}`).toString('base64');
    }
    compileAuthUrl(req, options, redirectUri) {
        const query = {
            app_token: this.clientId,
        };
        return options.handler
            .preAuthUrl(query, this, req)
            .then(() => `${this.authUrl}?${qs.stringify(query)}`);
    }
    extractCode(req) {
        return req.query['request_token'];
    }
    requestToken(code, redirect_uri) {
        return new Promise((resolve, reject) => {
            Wreck.post(this.tokenUrl, {
                json: true,
                payload: {
                    request_token: code,
                    app_token: this.clientId,
                    hash: this.hash,
                },
            }, (err, message, res) => {
                if (err) {
                    reject(err);
                    return;
                }
                if (res instanceof Buffer) {
                    res = res.toString('utf8');
                    try {
                        res = JSON.parse(res);
                    }
                    catch (_a) {
                        reject(new Error(res));
                        return;
                    }
                }
                if (res.error) {
                    const error = new Error(res.error_description);
                    error.name = err.error;
                    reject(error);
                    return;
                }
                resolve(res);
            });
        });
    }
    handleCode(h, options, redirectUri) {
        if (h.request.query['authToken']) {
            const data = {
                access_token: h.request.query['authToken'],
            };
            return options.handler.onLink({ provider: this, data }, h);
        }
        return super.handleCode(h, options, redirectUri);
    }
}
exports.SmashcastProvider = SmashcastProvider;
